@@ -6,12 +6,12 @@ Apache2::Controller::Session::Cookie - track a sessionid with a cookie in A2C
 
 =head1 VERSION
 
-Version 1.000.001 - FIRST RELEASE
+Version 1.000.010 - FIRST RELEASE
 
 =cut
 
 use version;
-our $VERSION = version->new('1.000.001');
+our $VERSION = version->new('1.000.010');
 
 =head1 SYNOPSIS
 
@@ -67,11 +67,11 @@ Readonly my $DEFAULT_COOKIE_NAME => 'A2CSession';
 
  my $sid = $self->get_session_id();
 
-Get the session id from the cookie.
+Get the session id from the cookie and verifies it.
 
-Sets C<<$r->notes->{session_id}>> to be the session id string.
+Sets C<< $r->notes->{session_id} >> to be the session id string.
 
-Sets C<<$r->pnotes->{session_cookie}>> to be the Apache2::Cookie object.
+Sets C<< $r->pnotes->{session_cookie} >> to be the Apache2::Cookie object.
 
 =cut
 
@@ -84,16 +84,33 @@ sub get_session_id {
     
     my $jar = $self->get_cookie_jar();
 
-    DEBUG("looking for cookie name '$cookie_name'");
+    DEBUG "looking for cookie name '$cookie_name'";
     my $cookie = $jar->cookies($cookie_name);
 
-    DEBUG($cookie ? "found cookie!" : "did not find cookie.");
+    DEBUG $cookie ? "found cookie!" : "did not find cookie.";
 
-    my $sid = $cookie ? $cookie->value : undef;
+    my ($sid, $sig) = $cookie ? $cookie->value : ();
+
+    return if !defined $sid || !defined $sig;
+
+    DEBUG sub { Dump({
+        sid_from_cookie => $sid,
+        sig_from_cookie => $sig,
+    }) };
+
+    # if the session_id does not pass signature, return nothing
+    my $valid_sig = $self->signature($sid);
+
+    if ($valid_sig ne $sig) {
+        WARN "signature validation failed";
+        return;
+    }
 
     my $r = $self->{r};
     $r->notes->{session_id} = $sid || '';
-    $r->pnotes->{session_cookie} = $cookie;
+
+    $self->{session_valid_sig} = $valid_sig;
+    $self->{session_cookie} = $cookie;
     
     return $sid;
 }
@@ -110,7 +127,6 @@ sub set_session_id {
     my ($self, $session_id) = @_;
     DEBUG("Setting session_id '$session_id'");
     my $r = $self->{r};
-    my $cookie = $r->pnotes->{session_cookie};
 
     my $directives = $self->get_directives();
 
@@ -120,12 +136,14 @@ sub set_session_id {
     DEBUG(sub {"Creating session cookie with opts:\n".Dump(\%copts)});
     my $name = delete $copts{name};
 
-    if (!$cookie) {
-        $cookie = Apache2::Cookie->new( $r,
-            -name           => $name,
-            -value          => $session_id,
-        );
-    }
+    my $cookie = $self->{session_cookie} || Apache2::Cookie->new( $r,
+        -name           => $name,
+        -value          => [ 
+            $session_id, 
+            ( $self->{valid_sig} || $self->signature($session_id) )
+        ],
+    );
+
     $cookie->$_($copts{$_}) for keys %copts;
 
     DEBUG("baking cookie '$cookie'");
@@ -143,6 +161,10 @@ sub set_session_id {
 
 L<Apache2::Controller::Session>
 
+L<Apache2::Controller::Directives/Apache2::Controller::Session>
+
+L<Apache2::Controller>
+
 L<Apache2::Cookie>
 
 =head1 AUTHOR
